package GUI;

import java.io.IOException;
import java.net.URL;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Comparator;
import java.util.List;
import java.util.ResourceBundle;
import java.util.Timer;
import java.util.TimerTask;

import CBR.CaseBase;
import CBR.Similarity;
import dataAcquisition.ConvertData;
import dataInterpretation.PrepareQueryThread;
import javafx.application.Platform;
import javafx.beans.property.DoubleProperty;
import javafx.beans.value.ChangeListener;
import javafx.beans.value.ObservableValue;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import javafx.event.ActionEvent;
import javafx.fxml.FXML;
import javafx.fxml.Initializable;
import javafx.scene.chart.BarChart;
import javafx.scene.chart.CategoryAxis;
import javafx.scene.chart.NumberAxis;
import javafx.scene.chart.XYChart;
import javafx.scene.control.Button;
import javafx.scene.control.Label;
import javafx.scene.control.ListView;
import javafx.scene.control.Slider;
import javafx.scene.control.TextArea;
import javafx.scene.control.TextField;

public class Controller implements Initializable {

	// initialise GUI elements and other needed stuff
	CaseBase casebase = new CaseBase();
	public static Similarity compareCase = new Similarity();
	ArrayList<Double> results = new ArrayList();
	public static String manualWork = "no";
	public static PrepareQueryThread R1 = new PrepareQueryThread("Thread 1");
	private ObservableList casesDisplay = FXCollections.observableArrayList();
	Timer timer = new Timer();
	public static ArrayList<String> caseObjects = new ArrayList<String>();

	@FXML
	private Slider thresholdSlider;

	@FXML
	private Label thresholdNumber;

	@FXML
	private ListView caseList;

	@FXML
	private Button btn;

	@FXML
	private Button manWork;

	@FXML
	private Label caseNum;

	@FXML
	private TextField titleText;

	@FXML
	private TextArea descriptionText;
	//
	@FXML
	private TextArea solutionText;

	public static double hoyde;
	public static double vekt;
	public static int alder;
	public static int iq;
	static double simThres;

	// act when slider is used. extract slider value
	@FXML
	public void handleSliderChange() {
		thresholdSlider.valueProperty().addListener(new ChangeListener() {
			@Override
			public void changed(ObservableValue observable, Object oldValue, Object newValue) {
				if (((double) (thresholdSlider.getValue()) == 0.5)) {
					thresholdNumber.textProperty().setValue(String.valueOf((double) (thresholdSlider.getValue())));
				} else {
					thresholdNumber.textProperty()
							.setValue(String.valueOf((double) (thresholdSlider.getValue() / 10.0)));
				}
			}
		});
	}

	// start the program when button is pushed
	@FXML
	public void handleButtonAction(ActionEvent e) throws Exception {
		run1();
	}

	// flip manual work button
	@FXML
	public void handleManualWork(ActionEvent e) {
		if (manualWork.equals("yes")) {
			manualWork = "no";
			System.out.println("Manual Work NO");
		} else {
			manualWork = "yes";
			System.out.println("Manual Work YES");
		}
	}

	// method for the things that happens when the program runs.
	// calls the similarity calculation method, visualises the elements and
	// resumes the thread from the other class
	void run1() throws Exception {

		casebase.fillCase();

		// call sim calc
		results = compareCase.globalSim(PrepareQueryThread.currentSpe, PrepareQueryThread.deviantData1,
				PrepareQueryThread.deviantData2, PrepareQueryThread.deviantData3, manualWork,
				PrepareQueryThread.shackleStatus, PrepareQueryThread.slopeDev, PrepareQueryThread.WeatherCon);

		simThres = Double.parseDouble(thresholdNumber.getText());

		// make list to visualise
		int overThresh = 0;

		for (int i = 0; i < results.size(); i++) {
			if (results.get(i) >= simThres) {
				overThresh++;
			}
		}

		Double[][] res = new Double[overThresh][2];

		int counter = 0;

		for (int i = 0; i < results.size(); i++) {
			if (results.get(i) >= simThres) {

				res[counter][0] = results.get(i);
				res[counter][1] = (double) i;

				counter++;
			}

		}

		int n = res.length;
		double temp1 = 0, temp2 = 0;

		for (int i = 0; i < n; i++) {
			for (int j = 1; j < (n - i); j++) {

				if (res[j - 1][0] > res[j][0]) {
					// swap the elements!
					temp1 = res[j - 1][0];
					res[j - 1][0] = res[j][0];
					res[j][0] = temp1;
					temp2 = res[j - 1][1];
					res[j - 1][1] = res[j][1];
					res[j][1] = temp2;
				}
			}
		}

		// generate objects to display in caseList
		casesDisplay.clear();
		for (int i = res.length - 1; i >= 0; i--) {
			casesDisplay.add("Case #" + (int) Math.round(res[i][1]) + " "
					+ casebase.CaseBase[(int) Math.round(res[i][1])][11] + ", has similarity = " + res[i][0]);
		}
		caseList.setItems(casesDisplay);
		caseObjects.clear();

		// Fill in descriptive info
		// solution.Description.add();

		if (res.length < 1) {
			caseNum.textProperty().set("No matching cases");
			titleText.setText("");
			descriptionText.setText("");
			solutionText.setText("");
		} else {
			caseNum.textProperty().setValue("Case #" + casebase.CaseBase[(int) Math.round(res[res.length - 1][1])][10]);
			titleText.setText(casebase.CaseBase[(int) Math.round(res[res.length - 1][1])][11]);

			descriptionText.setText(casebase.CaseBase[(int) Math.round(res[res.length - 1][1])][8]);
			solutionText.setText(casebase.CaseBase[(int) Math.round(res[res.length - 1][1])][9]);

		}

		R1.resume();

		timer.schedule(new TimerTask() {
			@Override
			public void run() {
				Platform.runLater(new Runnable() {
					public void run() {
						try {
							run1();
						} catch (Exception e) {
							// TODO Auto-generated catch block
							e.printStackTrace();
						}
					}
				});
			}
		}, 1000);
	}

	@Override
	public void initialize(URL arg0, ResourceBundle arg1) {
		// TODO Auto-generated method stub

	}

}
