package dataInterpretation;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.Closeable;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;

import weka.classifiers.Classifier;
import weka.classifiers.Evaluation;
import weka.classifiers.bayes.NaiveBayes;
import weka.classifiers.evaluation.NominalPrediction;
import weka.core.FastVector;
import weka.core.Instances;

public class ML {

	// read the model built in weka
	public static BufferedReader readDataFile(String filename) {
		BufferedReader inputReader = null;

		try {
			inputReader = new BufferedReader(new FileReader(filename));
		} catch (FileNotFoundException ex) {
			System.err.println("File not found: " + filename);
		}

		return inputReader;
	}

	// build the model with training set, test set and model
	public static Evaluation classify(Classifier model, Instances trainingSet, Instances testingSet) throws Exception {
		Evaluation evaluation = new Evaluation(trainingSet);

		model.buildClassifier(trainingSet);
		evaluation.evaluateModel(model, testingSet);

		return evaluation;
	}

	// perform the classification
	public static double predict(FastVector predictions) {

		NominalPrediction np = (NominalPrediction) predictions.elementAt(0);

		return np.predicted();
	}

	public static void close(Closeable stream) {
		try {
			if (stream != null) {
				stream.close();
			}
		} catch (IOException e) {
			// ...
		}
	}

	public static double run(double currSpeed, double currDir, int shackleID) throws Exception {

		String shackleString = System.getProperty("user.dir") + "/lib/" + shackleID + "_nominal.arff";

		BufferedReader datafile = readDataFile(shackleString);

		// read and change files depending on shackle ID
		try {
			String verify, putData;
			File file = new File(System.getProperty("user.dir") + "/lib/toClassify" + shackleID + ".arff");
			file.createNewFile();
			FileWriter fw2 = new FileWriter(file);
			BufferedWriter bw = new BufferedWriter(fw2);
			bw.write(
					"@relation '99875_test-weka.filters.unsupervised.attribute.MathExpression-Eifelse(A<0.4,ifelse(A<0.1,1,2),3)-R2,3-weka.filters.unsupervised.attribute.NumericToNominal-Rfirst'\n\n@attribute shackles {1,2,3}\n@attribute speed numeric\n@attribute direction numeric\n\n@data\n1,"
							+ currSpeed + "," + currDir);
			bw.flush();

			bw.close();

		} catch (IOException e) {
			e.printStackTrace();
		}

		BufferedReader testdata = readDataFile(
				System.getProperty("user.dir") + "/lib/toClassify" + shackleID + ".arff");
		Instances data = new Instances(datafile);
		Instances evaldata = new Instances(testdata);

		data.setClassIndex(0);
		evaldata.setClassIndex(0);

		// Use a set of classifiers
		Classifier[] models = { new NaiveBayes(), // a decision tree
		};

		// Collect every group of predictions for current model in a
		// FastVector
		FastVector predictions = new FastVector();

		// run methods::::
		Evaluation validation = classify(models[0], data, evaldata);

		predictions.appendElements(validation.predictions());

		double prediction = predict(predictions);

		// return the prediction
		return prediction;
	}

}